          section   TEXT

          xdef      __c2qdos
          xdef      __freestr

* This sub-routine converts a C string (pointed to by A1) into a QDOS string
* (which address is returned in A0). Note that it will allocate some memory
* into the heap (you will then have to de-allocate this memory once the QDOS
* string has been used). If the C string cannot fit into a QDOS string (more
* than 32764 characters), the routine returns with a "buffer full" error code.
* Modified registers: D0, D1, A0.

__c2qdos  MOVEQ     #2,D1                         Two bytes for the length word.
          MOVEA.L   A1,A0                         Copy C string address.
C2QDOS1   ADDQ.L    #1,D1                         One more character in string.
          TST.B     (A0)+                         Is it the end of the string ?
          BNE.S     C2QDOS1                       If not, repeat with next char.
          MOVEQ     #-5,D0                        Buffer full error code.
          CMPI.L    #32767,D1                     String length < 32765 bytes ?
          BPL.S     C2QDOS2                       If not, return with error.
          MOVEM.L   D1-D3/A1-A3,-(SP)             Save the context.
          BCLR      #0,D1                         Make D1 even.
          MOVEQ     #-1,D2                        Allocate for current job.
          MOVEQ     #$18,D0                       | Allocates D1 bytes into
          TRAP      #1                            |  the common heap.
          MOVEM.L   (SP)+,D1-D3/A1-A3             Restore the context.
          TST.L     D0                            Any error ?
          BNE.S     C2QDOS2                       If yes, return now.
          SUBQ.W    #3,D1                         Actual number of bytes in str.
          MOVE.W    D1,(A0)                       Store the length.
          BRA.S     C2QDOS4                       Start string copy.
C2QDOS3   MOVE.B    0(A1,D1.W),2(A0,D1.W)         Copy one character...
C2QDOS4   DBRA      D1,C2QDOS3                    ... until end of str.
C2QDOS2   TST.L     D0                            Set CCR accordingly to error.
          RTS                                     Return.

* This sub-routine de-allocate a heap space which address is held into A0.
* Modified registers: D1 to D3, A0 to A3.

__freestr MOVE.L    D0,-(SP)                      Save the error code.
          MOVEQ     #$19,D0                       | Free the memory reserved for
          TRAP      #1                            |  the filename by C2QDOS.
          MOVE.L    (SP)+,D0                      Restore the error code.
          RTS                                     Return.

          end
